/*
 *          Copyright (c) mjh-EDV Beratung, 1996-1999
 *     mjh-EDV Beratung - 63263 Neu-Isenburg - Rosenstrasse 12
 *          Tel +49 6102 328279 - Fax +49 6102 328278
 *                Email info@mjh.teddy-net.com
 *
 *    Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
 *
 *   $Id: peks-setup.c,v 1.7 1999/08/25 22:27:30 jordan Exp $
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *   PEKS - private exponent key stuff for Diffie Hellman and El Gamal
 */

#include "common-stuff.h"

#ifdef HAVE_SYS_SOCKET_H
# include <sys/socket.h>
#else
# include <socket.h> /* ????? */
#endif

#include "peks-internal.h"
#include "peks-setup.h"
#include "make-primes.h"
#include "version.h"
#include "peks-baseXX.h"
#include "peks-internal.h"
#include "messages.h"
#include "cipher.h"
#include "cbc-frame.h"
#include "rnd-pool.h"

#ifdef USE_CRIPPLED_ENCRYPTION_KEYS
#include "us_export_wizard.h"
#endif

/* ------------------------------------------------------------------------- *
 *                  private functions                                        *
 * ------------------------------------------------------------------------- */

static int
os_send
 (void     *fd, 
  void    *buf, 
  unsigned len, 
  int      flg)
{
  return OS_SEND ((int)fd, buf, len, flg) ;
}

static int
os_recv
 (void     *fd, 
  void    *buf, 
  unsigned len, 
  int      flg)
{
  return OS_RECV ((int)fd, buf, len, flg) ;
}

/* ------------------------------------------------------------------------- *
 *                  public functions: utilities                              *
 * ------------------------------------------------------------------------- */

char *
peks_version (void)
{
  return PEKS_VERSION 
# ifdef USE_PTHREADS
    ".pt"
# endif
# ifdef US_EXPORT_FINGERPRINT
    " (" US_EXPORT_BITS " bits: " US_EXPORT_FINGERPRINT ")" ;
# endif
    ;
}

unsigned
peks_keylen 
  (const peks_key *key)
{
  if (key == 0)
    return 0 ;

  /* return the number of bits in the prime module */
  return mpz_sizeinbase (key->modulus, 2) ;
}

/* ------------------------------------------------------------------------- *
 *                     public initialization functions                       *
 * ------------------------------------------------------------------------- */

peks_key *
dup_peks_key
  (const peks_key *old)
{
  peks_key *new = XMALLOC (sizeof (peks_key));

  mpz_init_set (new->modulus, old->modulus);
  mpz_init_set (new->private, old->private);

  new->generator  = old->generator ;

  POINT_OF_RANDOM_STACK (13);
  
  if (old->pubkey_str != 0)
    new->pubkey_str = XSTRDUP (old->pubkey_str);
  if (old->import_str != 0)
    new->import_str = XSTRDUP (old->import_str);
  if (old->challg_str != 0) 
    new->challg_str = XSTRDUP (old->challg_str) ;
  if (old->  host_str != 0) 
    new->  host_str = XSTRDUP (old->  host_str) ;

  new->socket = old->socket ;

  POINT_OF_RANDOM_VAR (new);

  return new;
}


peks_key *
new_peks_key
  (unsigned bits)
{
  void (*prt) (const char*) = xprint1 ;
  peks_key *k = XMALLOC (sizeof (peks_key));

  unsigned n;
  mpz_t G, PUB ;

  /* initialize */
  k->socket = -1 ;
  mpz_init (k->modulus) ;
  mpz_init (k->private) ;

  if (bits == 0) 
    bits = DEFAULT_PRIME_BITS ;
  if (bits < MINIMUM_PRIME_BITS)
    bits = MINIMUM_PRIME_BITS ;
  
  POINT_OF_RANDOM_VAR (k);

  /* generate prime module and generator  */
  n = get_generated_prime_module
    (&k->modulus, &k->generator, &k->private, bits, prt) ;

  POINT_OF_RANDOM_VAR (k);

  if (prt != 0) (*prt) ("\n") ; 
  if (n == 0) {
    errno = errnum (TOO_MANY_PRIME_TRIES) ;
    end_peks_key (k) ;
    return 0 ;
  }

  /* overwrite private with proper key */
  get_random_num (&k->private, bits, 0);	/* prv key as big as p */

  /* don't let the random number become too small */
  if (mpz_sizeinbase (k->private, 2) < bits-4)	/* set the msb to 1 */
    mpz_setbit (k->private, bits) ;		/* num |= 1 makes it big */

  /* calculate public key */
  mpz_init (PUB);
  mpz_init_set_ui (G, k->generator) ;
  mpz_powm (PUB, G, k->private, k->modulus);	/* pub key := g^a (mod p) */
 
  POINT_OF_RANDOM_VAR (PUB);

  /* clean up */
  mpz_clear (PUB) ;
  mpz_clear (G) ;
  return k;
}


int
peks_push_cipher
  (unsigned        fd,
   const char   *what,
   const char    *key,
   unsigned       len,
   unsigned is_sender)
{
  void *cbc, *cc, *fc ;
  cipher_desc *cp ;
  frame_desc  *fp ;
  int n, r ;
  size_t c_size ;
  int  (*c_open) (void*, void*, int(*)(void*,void*,unsigned,int),
		  cipher_desc*, frame_desc*, char[4]) ;
  int  (*c_rdwr) (void *, char *, unsigned, int);
  int  (*c_ioctl)(void *, io_ctrl_request, void*);
  void (*c_close)(void *) ;

  /* get new cipher and frame classes to be used, here */
  if (find_classes_by_name (&cc, &fc, what) < 0)
    return -1;

  /* create an cipher instance and a frame */
  if ((cp = (*(is_sender?create_encryption:create_decryption)) 
       (cc, key, len)) == 0)
    return -1;
  fp = create_frame (fc, key [len ? len-1 : 0]) ;
  
  /* push that new io layer */
  cbc_get_info (is_sender, &c_size, &c_open, &c_rdwr, &c_ioctl, &c_close);
  if ((cbc = io_push (fd, c_size, c_rdwr, c_ioctl, c_close, is_sender)) == 0)
    goto clean_up_return_error ;

  POINT_OF_RANDOM_STACK (11);
  
  /* initialize that io layer */
  if ((n = (*c_open) (cbc, (void*)fd, is_sender ? os_send : os_recv,
		      cp, fp, (char*)key + (len<5 ? 0 : len-5))) < 0)
    goto clean_up_return_error ;

# ifdef USE_ZLIB_COMPRESSION
  if (is_sender) {
    char *t = strchr (what, ':') ;
    if (t != 0 && t [1] &&
	/* set compression level */
	io_ctrl (fd, IO_COMPRESS_LEVEL, (r=t [1]-'0', &r), 1) < 0)
      goto clean_up_return_error ;
  }
# endif

  return n;

 clean_up_return_error: { 
    int err = errno ;
    destroy_cipher (cp);
    destroy_frame  (fp);
    errno = err ; 
  }
  return -1;
}


/* ------------------------------------------------------------------------- *
 *                     public destroy function                               *
 * ------------------------------------------------------------------------- */

void
end_peks_key_session
  (peks_key *key)
{
  if (key == 0)
    return ;

  key->socket = -1 ;

  if (key->import_str != 0) {
    XFREE (key->import_str) ;
    key->import_str = 0;
  }
  if (key->challg_str != 0) {
    XFREE (key->challg_str) ;
    key->challg_str = 0 ;
  }
  if (key->host_str != 0) {
    XFREE (key->host_str) ;
    key->host_str = 0 ;
  }
}


void
end_peks_key
  (peks_key *key)
{
  if (key == 0)
    return ;

  POINT_OF_RANDOM_VAR (key) ;

  mpz_clear (key->modulus) ;
  mpz_clear (key->private) ;

  POINT_OF_RANDOM_STACK (13);

  end_peks_key_session (key) ;

  if (key->pubkey_str != 0) 
    XFREE (key->pubkey_str);
  
  memset (key, 0, sizeof (*key)) ;
  XFREE (key) ;
}


